@@ -1,3 +1,4 @@
+import logging
 from itertools import combinations
 
 from .utils import (INF, get_bound, uniq, fsplit, drange, bbox2str, matrix2str, apply_matrix_pt,
@@ -616,6 +617,11 @@ class LTLayoutContainer(LTContainer):
             y1 = max(obj1.y1,obj2.y1)
             objs = set(plane.find((x0,y0,x1,y1)))
             return objs.difference((obj1,obj2))
+        if len(boxes) > 100:
+            # Grouping this many boxes would take too long and it doesn't make much sense to do so
+            # considering the type of grouping (nesting 2-sized subgroups) that is done here.
+            logging.warning("Too many boxes (%d) to group, skipping.", len(boxes))
+            return boxes
         # XXX this still takes O(n^2)  :(
         dists = []
         for obj1, obj2 in combinations(boxes, 2):
@@ -641,7 +647,7 @@ class LTLayoutContainer(LTContainer):
                 dists.append((0, dist(group,other), group, other))
             dists.sort(key=sortkey)
             plane.add(group)
-        assert len(plane) == 1
+        assert len(plane) in {0, 1}
         return list(plane)
     
     def analyze(self, laparams):
