@@ -4,13 +4,11 @@ import struct
 import hashlib as md5
 import logging
 
-from .psparser import PSStackParser, PSSyntaxError, PSEOF, literal_name, LIT, KWD, STRICT
+from .psparser import PSStackParser, PSSyntaxError, PSEOF, literal_name, LIT, KWD, handle_error
 from .pdftypes import (PDFException, PDFTypeError, PDFNotImplementedError, PDFStream, PDFObjRef,
     resolve1, decipher_all, int_value, str_value, list_value, dict_value, stream_value)
 from .arcfour import Arcfour
-from .utils import choplist, nunpack
-from .utils import decode_text, ObjIdRange
-
+from .utils import choplist, nunpack, decode_text, ObjIdRange
 
 ##  Exceptions
 ##
@@ -18,6 +16,7 @@ class PDFSyntaxError(PDFException): pass
 class PDFNoValidXRef(PDFSyntaxError): pass
 class PDFNoOutlines(PDFException): pass
 class PDFDestinationNotFound(PDFException): pass
+class PDFAlreadyParsed(PDFException): pass
 class PDFEncryptionError(PDFException): pass
 class PDFPasswordIncorrect(PDFEncryptionError): pass
 
@@ -247,7 +246,6 @@ class PDFPage:
 
 
 class PDFDocument:
-
     """PDFDocument object represents a PDF document.
 
     Since a PDF file can be very big, normally it is not loaded at
@@ -261,6 +259,8 @@ class PDFDocument:
       obj = doc.getobj(objid)
     
     """
+    
+    KEYWORD_OBJ = KWD('obj')
 
     def __init__(self, caching=True):
         self.caching = caching
@@ -272,7 +272,144 @@ class PDFDocument:
         self._parser = None
         self._cached_objs = {}
         self._parsed_objs = {}
-
+        self._parsed_everything = False
+    
+    def _parse_next_object(self, parser):
+        # This is a bit awkward and I suspect that it could be a lot more elegant, but it would
+        # require refactoring the parsing process and I don't want to do that yet.
+        stack = []
+        _, token = parser.nexttoken()
+        while token is not self.KEYWORD_OBJ:
+            stack.append(token)
+            _, token = parser.nexttoken()
+        objid = stack[-2]
+        genno = stack[-1]
+        _, obj = parser.nextobject()
+        return objid, genno, obj
+    
+    def _parse_objstream(self, stream):
+        # ObjStm have a special organization. First, the param "N" tells how many objs we have in
+        # there. Then, they start with a list of (objids, genno) pairs, and then the actual objects
+        # come in.
+        parser = PDFStreamParser(stream.get_data())
+        parser.set_document(self)
+        objcount = stream['N']
+        objids = []
+        for i in range(objcount):
+            _, objid = parser.nextobject()
+            _, genno = parser.nextobject()
+            objids.append(objid)
+        # Now we should be at the point where we read objects
+        for objid in objids:
+            _, obj = parser.nextobject()
+            self._cached_objs[objid] = obj
+    
+    def _parse_whole(self, parser):
+        while True:
+            try:
+                objid, genno, obj = self._parse_next_object(parser)
+                self._cached_objs[objid] = obj
+                if isinstance(obj, PDFStream) and obj.get('Type') is LITERAL_OBJSTM:
+                    obj.set_objid(objid, genno)
+                    self._parse_objstream(obj)
+            except PSEOF:
+                break
+    
+    def _parse_everything(self):
+        # Sometimes, we have malformed xref, but we still want to manage to read the PDF. In cases
+        # like these, the last resort is to read all objects at once so that our object reference
+        # can finally be resolved. This is slower than the normal method, so ony use this when the
+        # xref tables are corrupt/wrong/whatever.
+        if self._parsed_everything:
+            raise PDFAlreadyParsed()
+        parser = self._parser
+        parser.setpos(0)
+        parser.reset()
+        self._parse_whole(parser)
+        self._parsed_everything = True
+    
+    def _getobj(self, objid):
+        if not self.xrefs:
+            raise PDFException('PDFDocument is not initialized')
+        # logging.debug('getobj: objid=%r', objid)
+        if objid in self._cached_objs:
+            genno = 0
+            obj = self._cached_objs[objid]
+        else:
+            strmid, index = self.find_obj_ref(objid)
+            if index is None:
+                handle_error(PDFSyntaxError, 'Cannot locate objid=%r' % objid)
+                # return null for a nonexistent reference.
+                return None
+            if strmid:
+                stream = self.getobj(strmid)
+                if stream is None:
+                    return None
+                stream = stream_value(stream)
+                if stream.get('Type') is not LITERAL_OBJSTM:
+                    handle_error(PDFSyntaxError, 'Not a stream object: %r' % stream)
+                try:
+                    n = stream['N']
+                except KeyError:
+                    handle_error(PDFSyntaxError, 'N is not defined: %r' % stream)
+                    n = 0
+                if strmid in self._parsed_objs:
+                    objs = self._parsed_objs[strmid]
+                else:
+                    parser = PDFStreamParser(stream.get_data())
+                    parser.set_document(self)
+                    objs = []
+                    try:
+                        while True:
+                            _, obj = parser.nextobject()
+                            objs.append(obj)
+                    except PSEOF:
+                        pass
+                    if self.caching:
+                        self._parsed_objs[strmid] = objs
+                genno = 0
+                i = n*2+index
+                try:
+                    obj = objs[i]
+                except IndexError:
+                    raise PDFSyntaxError('Invalid object number: objid=%r' % (objid))
+                if isinstance(obj, PDFStream):
+                    obj.set_objid(objid, 0)
+            else:
+                try:
+                    self._parser.setpos(index)
+                except PSEOF:
+                    handle_error(PSEOF, 'Parser index out of bounds')
+                    return None
+                (_,objid1) = self._parser.nexttoken() # objid
+                (_,genno) = self._parser.nexttoken() # genno
+                (_,kwd) = self._parser.nexttoken()
+                # #### hack around malformed pdf files
+                #assert objid1 == objid, (objid, objid1)
+                if objid1 != objid:
+                    x = []
+                    while kwd is not self.KEYWORD_OBJ:
+                        (_,kwd) = self._parser.nexttoken()
+                        x.append(kwd)
+                    if x:
+                        objid1 = x[-2]
+                        genno = x[-1]
+                # #### end hack around malformed pdf files
+                if kwd is not self.KEYWORD_OBJ:
+                    raise PDFSyntaxError('Invalid object spec: offset=%r' % index)
+                try:
+                    (_,obj) = self._parser.nextobject()
+                    if isinstance(obj, PDFStream):
+                        obj.set_objid(objid, genno)
+                except PSEOF:
+                    return None
+            # logging.debug('register: objid=%r: %r', objid, obj)
+            if self.caching:
+                self._cached_objs[objid] = obj
+        if self.decipher:
+            obj = decipher_all(self.decipher, objid, genno, obj)
+        return obj
+    
     def set_parser(self, parser):
         "Set the document to use a given PDFParser object."
         if self._parser:
@@ -298,8 +435,7 @@ class PDFDocument:
         else:
             raise PDFSyntaxError('No /Root object! - Is this really a PDF?')
         if self.catalog.get('Type') is not LITERAL_CATALOG:
-            if STRICT:
-                raise PDFSyntaxError('Catalog not found!')
+            handle_error(PDFSyntaxError, 'Catalog not found!')
 
     # initialize(password='')
     #   Perform the initialization with a given password.
@@ -368,102 +504,50 @@ class PDFDocument:
         hash = md5.md5(key)
         key = hash.digest()[:min(len(key),16)]
         return Arcfour(key).process(data)
-
-    KEYWORD_OBJ = KWD('obj')
-    def getobj(self, objid):
-        if not self.xrefs:
-            raise PDFException('PDFDocument is not initialized')
-        # logging.debug('getobj: objid=%r', objid)
-        if objid in self._cached_objs:
-            genno = 0
-            obj = self._cached_objs[objid]
+    
+    def readobj(self):
+        """Read the next object at current position.
+        
+        The object doesn't have to start exactly where we are. We'll read the first
+        object that comes to us.
+        """
+        return self._parse_next_object(self._parser)
+    
+    def find_obj_ref(self, objid):
+        for xref in self.xrefs:
+            try:
+                strmid, index = xref.get_pos(objid)
+                return strmid, index
+            except KeyError:
+                pass
         else:
-            for xref in self.xrefs:
-                try:
-                    (strmid, index) = xref.get_pos(objid)
-                    break
-                except KeyError:
-                    pass
-            else:
-                if STRICT:
-                    raise PDFSyntaxError('Cannot locate objid=%r' % objid)
-                # return null for a nonexistent reference.
-                return None
-            if strmid:
-                stream = stream_value(self.getobj(strmid))
-                if stream.get('Type') is not LITERAL_OBJSTM:
-                    if STRICT:
-                        raise PDFSyntaxError('Not a stream object: %r' % stream)
-                try:
-                    n = stream['N']
-                except KeyError:
-                    if STRICT:
-                        raise PDFSyntaxError('N is not defined: %r' % stream)
-                    n = 0
-                if strmid in self._parsed_objs:
-                    objs = self._parsed_objs[strmid]
-                else:
-                    parser = PDFStreamParser(stream.get_data())
-                    parser.set_document(self)
-                    objs = []
-                    try:
-                        while 1:
-                            (_,obj) = parser.nextobject()
-                            objs.append(obj)
-                    except PSEOF:
-                        pass
-                    if self.caching:
-                        self._parsed_objs[strmid] = objs
-                genno = 0
-                i = n*2+index
-                try:
-                    obj = objs[i]
-                except IndexError:
-                    raise PDFSyntaxError('Invalid object number: objid=%r' % (objid))
-                if isinstance(obj, PDFStream):
-                    obj.set_objid(objid, 0)
-            else:
-                self._parser.setpos(index)
-                (_,objid1) = self._parser.nexttoken() # objid
-                (_,genno) = self._parser.nexttoken() # genno
-                (_,kwd) = self._parser.nexttoken()
-                # #### hack around malformed pdf files
-                #assert objid1 == objid, (objid, objid1)
-                if objid1 != objid:
-                    x = []
-                    while kwd is not self.KEYWORD_OBJ:
-                        (_,kwd) = self._parser.nexttoken()
-                        x.append(kwd)
-                    if x:
-                        objid1 = x[-2]
-                        genno = x[-1]
-                # #### end hack around malformed pdf files
-                if kwd is not self.KEYWORD_OBJ:
-                    raise PDFSyntaxError('Invalid object spec: offset=%r' % index)
-                try:
-                    (_,obj) = self._parser.nextobject()
-                    if isinstance(obj, PDFStream):
-                        obj.set_objid(objid, genno)
-                except PSEOF:
-                    return None
-            # logging.debug('register: objid=%r: %r', objid, obj)
-            if self.caching:
-                self._cached_objs[objid] = obj
-        if self.decipher:
-            obj = decipher_all(self.decipher, objid, genno, obj)
-        return obj
-
-    INHERITABLE_ATTRS = set(['Resources', 'MediaBox', 'CropBox', 'Rotate'])
+            # return null for a nonexistent reference.
+            return None, None
+    
+    def getobj(self, objid):
+        result = self._getobj(objid)
+        if result is None:
+            try:
+                self._parse_everything()
+                result = self._getobj(objid)
+            except PDFAlreadyParsed:
+                result = None
+        return result
+    
+    INHERITABLE_ATTRS = {'Resources', 'MediaBox', 'CropBox', 'Rotate'}
     def get_pages(self):
         if not self.xrefs:
             raise PDFException('PDFDocument is not initialized')
         def search(obj, parent):
-            if isinstance(obj, int):
-                objid = obj
-                tree = dict_value(self.getobj(objid)).copy()
-            else:
-                objid = obj.objid
-                tree = dict_value(obj).copy()
+            try:
+                if isinstance(obj, int):
+                    objid = obj
+                    tree = dict_value(self.getobj(objid), strict=True).copy()
+                else:
+                    objid = obj.objid
+                    tree = dict_value(obj, strict=True).copy()
+            except PDFTypeError:
+                return
             for (k,v) in parent.items():
                 if k in self.INHERITABLE_ATTRS and k not in tree:
                     tree[k] = v
@@ -599,16 +683,13 @@ class PDFParser(PSStackParser):
             try:
                 objlen = int_value(dic['Length'])
             except KeyError:
-                if STRICT:
-                    raise PDFSyntaxError('/Length is undefined: %r' % dic)
-                else:
-                    objlen = 0
+                handle_error(PDFSyntaxError, '/Length is undefined: %r' % dic)
+                objlen = 0
             self.setpos(pos)
             try:
                 (_, line) = self.nextline()  # 'stream'
             except PSEOF:
-                if STRICT:
-                    raise PDFSyntaxError('Unexpected EOF')
+                handle_error(PDFSyntaxError, 'Unexpected EOF')
                 return
             pos += len(line)
             endpos = pos + objlen
